@@ -64,7 +64,7 @@ static void free_val(char **val)
 
 static prelude_bool_t is_line_commented(const char *line) 
 {
-        line += strspn(line, " \t");
+        line += strspn(line, " \t\r");
         return ( *line == '#' ) ? TRUE : FALSE;
 }
 
@@ -77,7 +77,7 @@ static prelude_bool_t is_line_commented(const char *line)
  */
 static prelude_bool_t is_section(const char *line) 
 {
-        line += strspn(line, " \t\n");
+        line += strspn(line, " \t\r\n");
                 
         if ( *line == '[' && strchr(line, ']') )
                 return TRUE;
@@ -228,7 +228,8 @@ static int load_file_in_memory(config_t *cfg)
         fd = fopen(cfg->filename, "r");
         if ( ! fd ) {
                 prelude_string_destroy(out);
-                return prelude_error_from_errno(errno);
+                return prelude_error_verbose(prelude_error_code_from_errno(errno), "could not open '%s' for reading: %s",
+                                             cfg->filename, strerror(errno));
         }
         
         do {
@@ -278,7 +279,7 @@ static int strip_value(char **out, const char *in, size_t tlen)
         size_t slen, elen;
         prelude_bool_t have_start_quote = FALSE;
                 
-        in += slen = strspn(in, " \t");
+        in += slen = strspn(in, " \t\r");
         if ( *in == '"' ) {
                 in++; slen++;
                 have_start_quote = TRUE;
@@ -288,7 +289,7 @@ static int strip_value(char **out, const char *in, size_t tlen)
         if ( ! elen )
                 return 0;
         
-        while ( in[elen - 1] == ' ' || in[elen - 1] == '\t' )
+        while ( in[elen - 1] == ' ' || in[elen - 1] == '\t' || in[elen - 1] == '\r' )
                 elen--;
 
         if ( have_start_quote && elen ) {                
@@ -334,7 +335,7 @@ static int parse_buffer(const char *str, char **entry, char **value)
 }
 
 
-static int parse_section_buffer(const char *buf, char **entry, char **value)
+static int parse_section_buffer(const char *buf, char **entry, char **value, prelude_bool_t default_value)
 {
         int ret;
         char *ptr;
@@ -349,12 +350,9 @@ static int parse_section_buffer(const char *buf, char **entry, char **value)
         if ( ptr )
                 *ptr = ']';
         
-        if ( ret < 0 )
-                return ret;
-        
-        if ( ! *value )
+        if ( ! *value && default_value )
                 *value = strdup("default");
-        
+
         return ret;
 }
 
@@ -369,7 +367,7 @@ static int search_section(config_t *cfg, const char *section, unsigned int i)
         if ( ! cfg->content )
                 return -1;
 
-        ret = parse_section_buffer(section, &wentry, &wvalue);        
+        ret = parse_section_buffer(section, &wentry, &wvalue, TRUE);        
         if ( ret < 0 )
                 return ret;
                 
@@ -381,7 +379,7 @@ static int search_section(config_t *cfg, const char *section, unsigned int i)
                 if ( ! is_section(cfg->content[i]) )
                         continue;
                 
-                ret = parse_section_buffer(cfg->content[i], &entry, &value);
+                ret = parse_section_buffer(cfg->content[i], &entry, &value, TRUE);
                 if ( ret < 0 )
                         continue;
                 
@@ -392,7 +390,7 @@ static int search_section(config_t *cfg, const char *section, unsigned int i)
                         free(value);
                         continue;
                 }
-                
+
                 ret = strcasecmp(value, wvalue);
                 free(value);
                 
@@ -506,7 +504,8 @@ static int sync_and_free_file_content(config_t *cfg)
         
         fd = fopen(cfg->filename, "w");
         if ( ! fd ) 
-                return prelude_error_from_errno(errno);
+                return prelude_error_verbose(prelude_error_code_from_errno(errno), "could not open '%s' for writing: %s",
+                                             cfg->filename, strerror(errno));
         
         for ( i = 0; i < cfg->elements; i++ ) {
                 
@@ -729,16 +728,16 @@ int config_get_next(config_t *cfg, char **section, char **entry, char **value, u
                 return -1;
         
         while ( *line < cfg->elements ) {
-                
+                                
                 ptr = cfg->content[*line];
-                ptr += strspn(ptr, " \t");
+                ptr += strspn(ptr, " \t\r");
                 (*line)++;
-                
+
                 if ( ! *ptr || is_line_commented(ptr) )
                         continue;
                 
                 if ( is_section(ptr) )
-                        return parse_section_buffer(ptr, section, value);
+                        return parse_section_buffer(ptr, section, value, FALSE);
                 else          
                         return parse_buffer(ptr, entry, value);
         }
