@@ -59,6 +59,7 @@
 #define INDEX_FORBIDDEN (INT_MIN + 1)
 
 
+
 typedef struct idmef_path_element {
 
         int index;
@@ -81,6 +82,18 @@ struct idmef_path {
 };
 
 
+typedef struct {
+        int index;
+        idmef_path_t *path;
+        idmef_message_t *message;
+        prelude_bool_t reversed;
+        prelude_bool_t delete_list;
+} value_list_t;
+
+
+static int do_idmef_value_iterate(idmef_value_t *value, value_list_t *vl);
+
+
 static prelude_bool_t flush_cache = FALSE;
 static prelude_hash_t *cached_path = NULL;
 static pthread_mutex_t cached_path_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -194,11 +207,9 @@ static int idmef_path_get_list_internal(idmef_value_t **value_list,
                 if ( parent_class != -1 )
                         ret = idmef_path_get_internal(&value, path, depth, tmp, parent_class);
                 else {                        
-                        ret = idmef_value_new(&value, path->elem[depth - 1].value_type, tmp);                        
-                        if ( ret == 0 ) {
-                                idmef_value_dont_have_own_data(value);
+                        ret = idmef_value_new(&value, path->elem[depth - 1].value_type, tmp);
+                        if ( ret == 0 )
                                 ret = 1;
-                        }
                 }
 
                 if ( ret == 0 )
@@ -289,10 +300,111 @@ static int idmef_path_get_internal(idmef_value_t **value, idmef_path_t *path,
                 return 1;
         }
 
-        return (ret = idmef_value_new_class(value, parent_class, parent) < 0) ? ret : 1;
+        ret = (ret = idmef_value_new_class(value, parent_class, parent) < 0) ? ret : 1;
+        if ( ret == 1 )
+                idmef_class_ref(parent_class, parent);
+        
+        return ret;
 }
 
 
+static void delete_listed_child(void *parent, idmef_class_id_t class, idmef_path_element_t *elem)
+{
+        int ret;
+        void *obj;
+        prelude_list_t *head, *tmp, *bkp;
+
+        ret = idmef_class_get_child(parent, class, elem->position, (void *) &head);
+        if ( ret < 0 )
+                return;
+         
+         prelude_list_for_each_safe(head, tmp, bkp) {
+                 obj = prelude_linked_object_get_object(tmp);
+                 idmef_class_destroy(idmef_class_get_child_class(class, elem->position), obj);
+         }
+}
+
+
+
+static int _idmef_path_set(idmef_path_t *path, idmef_message_t *message, idmef_value_t *value, prelude_bool_t *delete_list)
+{
+        int i, ret, index;
+        void *ptr, *child;
+        idmef_value_type_id_t tid;
+        idmef_path_element_t *elem;
+        idmef_class_id_t class, parent_class;
+        
+        ptr = message;
+        parent_class = class = IDMEF_CLASS_ID_MESSAGE;
+        
+        for ( i = 0; i < path->depth; i++ ) {
+                elem = &path->elem[i];
+                
+                index = elem->index;
+
+                /*
+                 * We don't want to allow the user to set a single field within a listed
+                 * object without specifying the object index: alert.source.interface = "blah".
+                 */
+                if ( i + 1 < path->depth && index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) )
+                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_MISS_INDEX,
+                                                     "IDMEF path element '%s' need indexing",
+                                                     idmef_class_get_name(elem->class));
+
+                if ( index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) ) {
+                        index = IDMEF_LIST_APPEND;
+                        if ( *delete_list ) {
+                                *delete_list = FALSE;
+                                delete_listed_child(ptr, class, elem);
+                        }
+                }
+                
+                ret = idmef_class_new_child(ptr, class, elem->position, index, &child);
+                if ( ret < 0 )
+                        return ret;
+                
+                ptr = child;
+                parent_class = class;
+                
+                class = idmef_class_get_child_class(class, elem->position);                
+                assert( ! (class < 0 && i < path->depth - 1) );
+        }
+
+        tid = idmef_class_get_child_value_type(parent_class, path->elem[path->depth - 1].position);
+        if ( tid != idmef_value_get_type(value) )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "IDMEF path copy of type '%s' to '%s' is forbidden",
+                                             idmef_value_type_to_string(idmef_value_get_type(value)),
+                                             idmef_value_type_to_string(tid));
+        
+        return idmef_value_get(value, ptr);
+}
+
+
+static int unroll_listed_value(idmef_value_t *value, void *extra)
+{
+        value_list_t *val = extra;
+        
+        if ( idmef_value_is_list(value) )
+                return do_idmef_value_iterate(value, extra);
+
+        return _idmef_path_set(val->path, val->message, value, &val->delete_list);
+}
+
+
+static int do_idmef_value_iterate(idmef_value_t *value, value_list_t *vl)
+{
+        int ret;
+
+        if ( vl->reversed )
+                ret = idmef_value_iterate_reversed(value, unroll_listed_value, vl);
+        else
+                ret = idmef_value_iterate(value, unroll_listed_value, vl);
+
+        return ret;
+}
+
+
+
 
 /**
  * idmef_path_get:
@@ -324,41 +436,26 @@ int idmef_path_get(idmef_path_t *path, idmef_message_t *message, idmef_value_t *
  */
 int idmef_path_set(idmef_path_t *path, idmef_message_t *message, idmef_value_t *value)
 {
-        int i, ret;
-        void *ptr, *child;
-        idmef_value_type_id_t tid;
-        idmef_path_element_t *elem;
-        idmef_class_id_t class, parent_class;
+        prelude_bool_t delete_list = TRUE;
+        idmef_path_element_t *elem = &path->elem[path->depth - 1];
         
-        ptr = message;
-        parent_class = class = IDMEF_CLASS_ID_MESSAGE;
-        
-        for ( i = 0; i < path->depth; i++ ) {
-                elem = &path->elem[i];
+        /*
+         * Allow raw list copy (example: alert.source = alert.source,
+         * alert.source(>>) = alert.source, alert.source(<<) = alert.source.
+         */
+        if ( idmef_value_is_list(value) ) {
+                value_list_t vl;
                 
-                if ( elem->index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) )
-                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_MISS_INDEX,
-                                                     "IDMEF path element '%s' need indexing",
-                                                     idmef_class_get_name(elem->class));
-                
-                ret = idmef_class_new_child(ptr, class, elem->position, elem->index, &child);
-                if ( ret < 0 )
-                        return ret;
-                
-                ptr = child;
-                parent_class = class;
-                
-                class = idmef_class_get_child_class(class, elem->position);                
-                if ( class < 0 && i < path->depth - 1 )
-                        abort();
+                vl.path = path;
+                vl.message = message;
+                vl.delete_list = TRUE;
+                vl.reversed = (elem->index == IDMEF_LIST_PREPEND) ? TRUE : FALSE;
+                                        
+                return do_idmef_value_iterate(value, &vl);
         }
 
-        tid = idmef_class_get_child_value_type(parent_class, path->elem[path->depth - 1].position);
-        
-        if ( idmef_value_get_type(value) != tid )
-                abort();
-        
-        return idmef_value_get(value, ptr);
+                
+        return _idmef_path_set(path, message, value, &delete_list);
 }
 
 
@@ -467,12 +564,13 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
                         return child;
                 
                 path->elem[depth].position = child;
-                
+
+                if ( ptr2 )
+                        *ptr2 = '(';
+
                 if ( index == INDEX_UNDEFINED )
                         path->elem[depth].index = idmef_class_is_child_list(class, child) ? INDEX_UNDEFINED : INDEX_FORBIDDEN;
-                else {
-                        *ptr2 = '(';
-                        
+                else {                        
                         if ( ! idmef_class_is_child_list(class, child) )
                                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN,
                                                              "Invalid IDMEF path element '%s': indexing not supported", ptr);
