@@ -157,7 +157,6 @@ GENERIC_TWO_BASES_RW_FUNC("%u", "%x", uint32, uint32_t)
 GENERIC_TWO_BASES_RW_FUNC("%" PRELUDE_PRId64, "%" PRELUDE_PRIx64, int64, int64_t)
 GENERIC_TWO_BASES_RW_FUNC("%" PRELUDE_PRIu64, "%" PRELUDE_PRIx64, uint64, uint64_t)
 
-GENERIC_ONE_BASE_RW_FUNC("%d", "%d", enum, enum)
 GENERIC_ONE_BASE_RW_FUNC("%f", "%f", float, float)
 GENERIC_ONE_BASE_RW_FUNC("%lf", "%f", double, double)
 
@@ -205,6 +204,37 @@ static int generic_compare(const idmef_value_type_t *t1, const idmef_value_type_
 
 
 
+/*
+ * Enum specific
+ */
+static int enum_copy(const idmef_value_type_t *src, void *dst, size_t size)
+{
+        *(int *)dst = src->data.enum_val.value;
+        return 0;
+}
+
+
+static int enum_read(idmef_value_type_t *dst, const char *buf)
+{
+        int ret;
+        
+        ret = sscanf(buf, "%d", &(dst)->data.enum_val.value);
+
+        return (ret == 1) ? 0 : prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, "Reading enum value failed");
+}
+
+
+
+static int enum_write(const idmef_value_type_t *src, prelude_string_t *out)
+{
+        const char *str;
+
+        str = idmef_class_enum_to_string(src->data.enum_val.class_id, src->data.enum_val.value);
+
+        return prelude_string_cat(out, str);
+}
+
+
 
 /*
  * time specific function.
@@ -405,6 +435,28 @@ static void data_destroy(idmef_value_type_t *type)
 
 
 
+/*
+ *
+ */
+static int class_copy(const idmef_value_type_t *src, void *dst, size_t size)
+{
+        return idmef_class_copy(src->data.class_val.class_id, src->data.class_val.object, dst);
+}
+
+
+static int class_clone(const idmef_value_type_t *src, idmef_value_type_t *dst, size_t size)
+{
+        dst->data.class_val.class_id = src->data.class_val.class_id;
+        return idmef_class_clone(src->data.class_val.class_id, src->data.class_val.object, &dst->data.class_val.object);
+}
+
+                        
+static void class_destroy(idmef_value_type_t *type)
+{
+        idmef_class_destroy(type->data.class_val.class_id, type->data.class_val.object);
+}
+
+
 static const idmef_value_type_operation_t ops_tbl[] = {
         { "unknown", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL                     },
         { "int8", sizeof(int8_t), INTEGER_OPERATOR, generic_copy,
@@ -433,10 +485,10 @@ static const idmef_value_type_operation_t ops_tbl[] = {
           time_clone, time_destroy, time_compare, time_read, time_write           }, 
         { "data", 0, DATA_OPERATOR, data_copy,
           data_clone, data_destroy, data_compare, data_read, data_write           },
-        { "enum", sizeof(idmef_value_type_id_t), INTEGER_OPERATOR, generic_copy,
+        { "enum", sizeof(idmef_value_type_enum_t), INTEGER_OPERATOR, enum_copy,
           generic_clone, NULL, generic_compare, enum_read, enum_write,            },
         { "list", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL                        },
-        { "class", 0, CLASS_OPERATOR, NULL, NULL, NULL, NULL, NULL, NULL          },
+        { "class", 0, CLASS_OPERATOR, class_copy, class_clone, class_destroy, NULL, NULL, NULL },
 };
 
 
@@ -451,13 +503,13 @@ static int is_type_valid(idmef_value_type_id_t type)
 
 
 
-static const char *type_id_to_string(idmef_value_type_id_t type)
+const char *idmef_value_type_to_string(idmef_value_type_id_t type)
 {
         return ops_tbl[type].name;
-        
 }
 
 
+
 int idmef_value_type_clone(const idmef_value_type_t *src, idmef_value_type_t *dst)
 {
         int ret;
@@ -471,7 +523,7 @@ int idmef_value_type_clone(const idmef_value_type_t *src, idmef_value_type_t *ds
         if ( ! ops_tbl[dst->id].clone )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_CLONE_UNAVAILABLE,
                                              "Object type '%s' does not support clone operation",
-                                             type_id_to_string(dst->id));
+                                             idmef_value_type_to_string(dst->id));
         
         return ops_tbl[dst->id].clone(src, dst, ops_tbl[dst->id].len);
 }
@@ -490,7 +542,7 @@ int idmef_value_type_copy(const idmef_value_type_t *src, void *dst)
         if ( ! ops_tbl[src->id].copy )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_COPY_UNAVAILABLE,
                                              "Object type '%s' does not support copy operation",
-                                             type_id_to_string(src->id));
+                                             idmef_value_type_to_string(src->id));
         
         return ops_tbl[src->id].copy(src, dst, ops_tbl[src->id].len);
 }
@@ -516,7 +568,7 @@ int idmef_value_type_compare(const idmef_value_type_t *type1,
         if ( ! ops_tbl[type1->id].compare )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_COMPARE_UNAVAILABLE,
                                              "Object type '%s' does not support compare operation",
-                                             type_id_to_string(type1->id));
+                                             idmef_value_type_to_string(type1->id));
         
         ret = ops_tbl[type1->id].compare(type1, type2, ops_tbl[type1->id].len, op & ~IDMEF_CRITERION_OPERATOR_NOT);
         if ( op & IDMEF_CRITERION_OPERATOR_NOT )
@@ -539,7 +591,7 @@ int idmef_value_type_read(idmef_value_type_t *dst, const char *buf)
         if ( ! ops_tbl[dst->id].read )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_READ_UNAVAILABLE,
                                              "Object type '%s' does not support read operation",
-                                             type_id_to_string(dst->id));
+                                             idmef_value_type_to_string(dst->id));
         
         ret = ops_tbl[dst->id].read(dst, buf);
         return (ret < 0) ? ret : 0;
@@ -559,7 +611,7 @@ int idmef_value_type_write(const idmef_value_type_t *src, prelude_string_t *out)
         if ( ! ops_tbl[src->id].write )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_WRITE_UNAVAILABLE,
                                              "Object type '%s' does not support write operation",
-                                             type_id_to_string(src->id));
+                                             idmef_value_type_to_string(src->id));
         
         return ops_tbl[src->id].write(src, out);
 }
@@ -596,7 +648,5 @@ int idmef_value_type_check_operator(const idmef_value_type_t *type, idmef_criter
         
         return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERION_UNSUPPORTED_OPERATOR,
                                      "Object type '%s' does not support operator '%s'",
-                                     type_id_to_string(type->id), idmef_criterion_operator_to_string(op));
-        
-        return prelude_error(PRELUDE_ERROR_IDMEF_CRITERION_UNSUPPORTED_OPERATOR);
+                                     idmef_value_type_to_string(type->id), idmef_criterion_operator_to_string(op));
 }
